
/*******************************************************************************

  Timer Programvel 

  Com PIC16F628A  e display LCD  16 x 2
  

  Autor: Cludio Lrios

  Data: 10/08/2015

  Clock: 4MHz
  
  Temporizador programvel mx. ligado = 59 minutos e 59 segundos
  Apresenta mensagem: 'Timer On','Timer Off','Timer Pause'
  Produz beep ao termino da temporizao para alertar o usurio
  
  Use CCS C compiler
  Obs. Arquivo para fins didticos apenas.
  Este arquivo  parte integrante do blog Lrios.Tecnologia.WS

*******************************************************************************/

#include <16F628A.h>
#use delay(clock=4000000)//xtal de 4mhz externo nos pinos 15 e 16
#fuses NOWDT,XT, PUT, NOPROTECT, BROWNOUT, MCLR, NOLVP, NOCPD
#zero_ram//zera no nicio
#use fast_io(a)//modo rpido
#use fast_io(b)//modo rpido
//bytes
#byte trisb = 0x86
#byte trisa = 0x85
#byte tmr0  = 0x01
#byte portb = 0x06
#byte porta = 0x05

// lcd
#bit   en     = 0x06.2 //rb2  lcd 'enable' pino 8
#bit   rs     = 0x06.3 //rb3  lcd 'rs' - comando/dado pino 9
#bit   db4    = 0x06.4 //rb4  db4 pino 10
#bit   db5    = 0x06.5 //rb5  db5 pino 11
#bit   db6    = 0x06.6 //rb6  db6 pino 12
#bit   db7    = 0x06.7 //rb7  db7 pino 13
#bit   tris_en     = 0x86.2 //rb2  lcd 'enable' pino 8
#bit   tris_rs     = 0x86.3 //rb3  lcd 'rs' - comando/dado pino 9
#bit   tris_db4    = 0x86.4 //rb4  dir db4 pino 10
#bit   tris_db5    = 0x86.5 //rb5  dir db5 pino 11
#bit   tris_db6    = 0x86.6 //rb6  dir db6 pino 12
#bit   tris_db7    = 0x86.7 //rb7  dir db7 pino 13

//bits
#bit rele    = 0x5.3 //rele pino 3
#bit sw_min  = 0x5.2  //ajuste dos minutos
#bit sw_stp  = 0x5.1  //boto de start-pause
#bit sw_seg  = 0x5.0  //ajustes dos segundos
#bit beep    = 0x6.1  //saida do beep (piezoeletrico simples)
#bit int_tmr0_on  = 0x0b.5 //flag on/off de interrupao do timer 0


//variveis globais
int8 cnt, minutos, segundos;
int1 f_trava1,f_bipar,f_dl,f_pause;

//==============================================================================
//                   ROTINAS PARA LCD
//==============================================================================
//clock para lcd
void lcd_clk(void){
 en=1;
 delay_us(10);//
 en=0;

 }

//envia nible para lcd
 void env_nib(int8 b){

 if(b&0x10)db4=1; else db4=0;
 if(b&0x20)db5=1; else db5=0;
 if(b&0x40)db6=1; else db6=0;
 if(b&0x80)db7=1; else db7=0;
 lcd_clk();
 }

 //envia comando para lcd
 void env_lcd_c(int8 val){
 rs=0;
 env_nib(val);
 env_nib(swap(val));
 delay_ms(10);
}

//envia dados para lcd
 void env_lcd_d(int8 val){
 rs=1;
 env_nib(val);
 env_nib(swap(val));
 }

//inicializao do lcd
void lcd_init(void){
 int8 a;

  rs=0;
  tris_en=0;//direo dos pinos como saida
  tris_rs=0;
  tris_db4=0;
  tris_db5=0;
  tris_db6=0;
  tris_db7=0;
  db4=0;//situao inicial resetados
  db5=0;
  db6=0;
  db7=0;
  en=0;
  delay_ms(50);//delay inicial
  db4=1; db5=1;// setados
  for(a=0;a<3;a++) {lcd_clk(); delay_ms(5);}// envia 3,3,3
  db4=0;//envia 2
  lcd_clk();//pulso em 'en'
  delay_ms(5);
  env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
  env_lcd_c(0x06);// no shift
  env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  env_lcd_c(0x01);//limpa tela
  delay_ms(10);

}
//-------------------------FIM DAS ROTINAS DE LCD--------------------------------


//rotina de produo de som no disco piezoeletrico
 void bipar() {
   int16 c;
   int8 a;
   f_bipar=0;
   for(a=0;a<5;a++){
    for(c=0; c<2500; c++) {
    
      beep=1;
      delay_us(193); //ajuste conforme freq. ressonancia do piezoeletrico
      beep=0;
      delay_us(11);
    }
    delay_ms(100);//delay entre beeps
  }
 }


 //=============================================================================
 //    ROTINA DE INTERRUPO DO TIMER 0  (CADA 8MS)
 //=============================================================================
#int_timer0
void interrupt()
{
   TMR0 = get_timer0()+131; //carrega TMR0 com o valor
   if(++cnt>=125){ //8ms * 125 = 1s
         cnt=0;
         if(--segundos==0xff){
           segundos=59;
           if( minutos>0) --minutos;
           }
         if(!minutos&&!segundos) {
               rele=0;//desliga o rele de sada
               disable_interrupts(int_timer0);//para a contagem
               f_bipar=1;//sinaliza para bipar (fim da contagem)
               f_pause=0;
        }
    }
}

//=============================================================================
 //    ROTINA DE MAIN
 //=============================================================================


void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_64);//timer 0 prescaller/ 64
   setup_timer_1(T1_DISABLED);//desliga tmr1
   setup_timer_2(T2_DISABLED,0,1);//desliga tmr2
   setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
   setup_vref(FALSE);//desliga vref
   enable_interrupts(global);//ativa chave de interrupes
   TMR0 = 131; //Carrega TMR0 com o valor
   trisb = 0;//Define PORTb como saidas
   trisa = 0b11100111;//Define PORTa in/out
   portb=0;//reseta portb
   rele= 0; //desativa rele de saida e desliga a carga
   Lcd_Init(); //Inicializa o display
   env_lcd_c(0x80);//linha 1
   printf(env_lcd_d("Timer Eletronico"));// nome do aparelho
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d("Larios.Tecnol.Ws"));//blog
   delay_ms(2000);//tempo de apresentao
   env_lcd_c(1);//limpa tela

 //=============================================================================
 //   LAO PRINCIPAL DE REPETIO
 //=============================================================================

   for(;;){

    
       if(!rele){ //ajustes somente em 'pausa'
       if(sw_seg){ // boto de ajuste de segundos se pressionado comea incrementar
           if(++segundos>59) segundos=0;
           delay_ms(250);//velocidade de incremento
           if(!sw_seg) if(--segundos==0xff) segundos=59;;//se boto for solto durante o delay acima
           f_pause=0;//muda de 'pause' para 'off'
           }


        if(sw_min){ //boto ajuste de minutos se pressionado comea incrementar
           if(++minutos>59) minutos=0;
            delay_ms(250);
            if(!sw_min) if(--minutos==0xff) minutos=59;//se boto for solto durante o delay acima
            f_pause=0;//muda de "pause' para 'off'
         }

       }

      if(sw_stp&&!f_trava1) {//boto start/pause  (1 toque 'liga' outro 'pausa')
         f_trava1=1;
        if((segundos>0)||(minutos>0)){//liga menos com '00:00'
         if(!rele){//liga a contagem
            rele=1;//liga rele e aciona a carga
            f_pause=1;
            enable_interrupts(int_timer0);
            }
         else {
           disable_interrupts(int_timer0);//para a contagem
           rele=0; //desliga na pausa
         }
       }
      }
      if(!sw_stp) f_trava1=0; //reseta a trava
      
      env_lcd_c(0x83);//linha 1 p3
      if(int_tmr0_on){
        printf(env_lcd_d("Timer On   "));//imprime mensagem 'Timer on '
        f_dl=0;
      }
      else{
        if(!f_pause){
        printf(env_lcd_d("Timer Off  "));//imprime mensagem 'Timer on'
        f_dl=1;//autoriza o bip somente aps imprimir o acima
        }
        else{
           printf(env_lcd_d("Timer Pause"));//imprime mensagem 'Timer on '
           f_dl=0;
        }
      }
     
      
      env_lcd_c(0xc5);//linha 2 p5
      env_lcd_d(minutos/10+'0');//envia dezenas de minutos
      env_lcd_d(minutos%10+'0');//envia unidades de minutos
      if( int_tmr0_on && (cnt<20)) //pisca ':' muito brevemente
          { env_lcd_d(' ');}//apaga ':'
      else
        { env_lcd_d(':');}//mostra ':'
      env_lcd_d(segundos/10+'0');//envia dezenas de segundos
      env_lcd_d(segundos%10+'0');//envia unidades de segundos
      if((f_bipar)&&(f_dl)) {f_dl=0; bipar();} //beep de fim da temporizao
      delay_ms(1);//delay do loop principal
   }


}// for(;;)


//======================Fim do Programa=========================================









